/* -------------------------------------------------------
                      avr_rtc_demo.c

     Softwareuhr, realisiert mittels Timer2 der durch
     einen 32,768 kHz Uhrenquartz getaktet ist

     MCU   :  ATMega168
     Takt  :  Coretakt 8 MHz intern
              RTC-Takt 32,768 kHz Uhrenquarz

     04.05.2021  R. Seelig
   ------------------------------------------------------ */


#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "avr_gpio.h"
#include "my_printf.h"
#include "uart_all.h"
#include "avr_rtc.h"

#define printf    my_printf
#define delay     _delay_ms

// PB5 ist die Onboard-LED auf einem Arduino UNO

#define led_init()     PB5_output_init()
#define led_clr()      PB5_set()
#define led_set()      PB5_clr()

/* --------------------------------------------------
                     dez2bcd

      wandelt eine dezimale Zahl in eine BCD
      Bsp: value = 45
      Rueckgabe    0x45
   -------------------------------------------------- */
uint8_t dez2bcd(uint8_t value)
{
  uint8_t hiz,loz,c;

  hiz= value / 10;
  loz= (value -(hiz*10));
  c= (hiz << 4) | loz;
  return c;
}

/* --------------------------------------------------
     UHR_STELLEN

     die Uhr benutzerabgefragt stellen
   -------------------------------------------------- */
void uhr_stellen(void)
{
  int       i;

  do
  {
    printf("\n\rTag    : ");
    i= uart_readint();
  } while ( !((i > -1) && (i < 32)));
  day= i;

  do
  {
    printf("\n\rMonat  : ");
    i= uart_readint();
  } while ( !((i > -1) && (i < 13)));
  month= i;

  do
  {
    printf("\n\rJahr   : ");
    i= uart_readint();
  } while ( !((i > -1) && (i < 100)));
  year= 2000+i;

  do
  {
    printf("\n\rStunde : ");
    i= uart_readint();
  } while ( !((i > -1) && (i < 24)));
  std= i;

  do
  {
    printf("\n\rMinute : ");
    i= uart_readint();
  } while ( !((i > -1) && (i < 60)));
  min= i;

  do
  {
    printf("\n\rSekunde: ");
    i= uart_readint();
  } while ( !((i > -1) && (i < 60)));
  sek= i;

}

/* --------------------------------------------------
     my_putchar

     Diese Funktion wird von < my_printf > zur Ausgabe
     von Zeichen aufgerufen. Hier ist eine Hardware-
     zeichenausgabefunktion anzugeben, mit der
     my_printf zusammen arbeiten soll.
   -------------------------------------------------- */
void my_putchar(char ch)
{
  uart_putchar(ch);
}


/* ----------------------------------------------------------------------------------
                                           MAIN
   ---------------------------------------------------------------------------------- */

void main(void)
{
  uint8_t   oldsek;
  int       wt;

  char wtag[][3] = {"So","Mo","Di","Mi","Do","Fr","Sa"};

  led_init();
  uart_init();                  // serielle Schnittstelle
  timer2_init();                // Timer2 - Interrupt

  std= 0; min= 0; sek= 0; hsek= 0, year= 2021; month= 5; day= 4;
  printf("\n\n\rATmegaxx8 - Softwareuhr\n\r");
  printf(      "---------------------------------------\n\r");
  printf(      "RTC-clock: 32768 Hz | UART: 19200Bd 8N1\n\r");
  printf("\n\n\rUhr stellen:\n\r");

  uhr_stellen();

  printf("\n\n\r");
  oldsek= sek-1;

  while(1)
  {
    while(sek == oldsek)                // warten bis Sekunde "weiterspringt"
    {
      if (hsek== 1) led_set(); else led_clr();
    }

    oldsek= sek;
    wt= getwtag(day, month, year);

    // Umrechnung in BCD-Zahlen und Darstellung als Hexwerte, damit
    // eine fuehrende Null mit ausgegeben wird (ist den eingeschraenkten
    // Moeglichkeiten von - my_printf - geschuldet )
    printf("   %s:  %x.%x.%d  %x.%x:%x  \r", wtag[wt],           \
               dez2bcd(day), dez2bcd(month), year,               \
               dez2bcd(std), dez2bcd(min), dez2bcd(sek));
  }
}
