/* -----------------------------------------------------------
                        crystal_int.c

   Versuche mit 32,768 kHz Uhrenquarz

   04.05.2021 R. Seelig
   ---------------------------------------------------------- */


#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "avr_gpio.h"


// PB5 ist die Onboard-LED auf einem Arduino UNO

#define led_init()     PB5_output_init()
#define led_clr()      PB5_set()
#define led_set()      PB5_clr()

#define delay          _delay_ms

#define blkspeed_high  500
#define blkspeed_low   500


volatile uint8_t hsek    = 0;
volatile uint8_t sekunde = 0;
volatile uint8_t minute  = 11;
volatile uint8_t stunde  = 11;


/* -----------------------------------------------------------
                    TIMER2 comp A ISR
                    Interruptvektor
   ----------------------------------------------------------- */
ISR (TIMER2_COMPA_vect)
{
    TCCR2B = TCCR2B;              
    hsek++;
    if (hsek== 2)
    {
      sekunde++;
      hsek= 0;
    }  
    if(sekunde == 60)
    {
      minute++;
      sekunde = 0;
    }
    if(minute == 60)
    {
      stunde++;
      minute = 0;
    }
    if(stunde == 24)
    {
      stunde = 0;
    }
    // Dummy-Zugriff
    TCCR2B = TCCR2B;             
    
    // warten bis ein schreiben erfolgt ist
    while(ASSR & ((1 << TCN2UB) | (1 << OCR2AUB) | (1 << OCR2BUB) |
                  (1 << TCR2AUB) | (1 << TCR2BUB)));
}

/* -----------------------------------------------------------
                        timer2_init                    
                        
     initialisiert Timer2 fuer einen 32,768 kHz Uhren-
     quarz fuer einen Interruptintervall = 1 Sekunde                        
   ----------------------------------------------------------- */
void timer2_init(void)   
{
  GTCCR |= (1 << TSM) | (1 << PSRASY);    // Timer stop, Prescaler Reset
  ASSR |= (1 << AS2);                     // Asynchron Mode
  TCCR2A = (1 << WGM21);                  // CTC Modus
  TCCR2B |= (1 << CS22) | (1 << CS21);    // Prescaler 256
  
  // 32768 / 256 / 0.5 = 64                Intervall = 0.5s
  
  OCR2A = 64 - 1;
  TIMSK2 |= (1 << OCIE2A);                // enable compare interrupt
  GTCCR &= ~(1 << TSM);                   // Timer starten

  sei();                                  // grundsaetzlich Interrupts zulassen
}

/* -----------------------------------------------------------
                              MAIN                    
   ----------------------------------------------------------- */
int main(void)
{

  timer2_init();
  led_init();

  while(1)
  {
/*    led_set();
    delay(blkspeed_high);
    led_clr();
    delay(blkspeed_low);
*/
    if (sekunde & 0x01) led_set(); else led_clr();
  }
}
