/* -----------------------------------------------------------------------
                                ds18b20.h

     Header fuer Softwaremodul zum Auslesen eines DS18B20
     Temperatursensors mittels bitbangingrealisierten
     OneWire Bus.

     Rudimentaere Funktionen bei denen es NICHT moeglich ist,
     mehrere Sensoren an den Bus anzuschliessen.


     MCU     : grundsaetzlich alle AVR, momentan getestet mit
               ATMega328p
     F_CPU   :

     28.06.2024   R.Seelig
   -----------------------------------------------------------------------*/

#ifndef in_ds18b20
  #define in_ds18b20

  #include <avr/io.h>

  // -----------------------------------------------------------------------
  //   Datenanschlusspin des DS18B20
  // -----------------------------------------------------------------------

  // Anschluss an PC0
  #define ONEWIRE_PORT     C
  #define ONEWIRE_CONNECT  0


  // -----------------------------------------------------------------------
  //   da das Auslesen des Sensors sehr zeitkritisch ist, sollte / duerfen
  //   die Funktionen nicht von einem Interrupt unterbrochen werden. Wird
  //   dieser Schalter auf 1 gesetzt, werden global alle Interrupts vor
  //   ausfuehren der Lesefunktion gesperrt und nach Beendigung der Lese-
  //   wieder freigegeben.
  // -----------------------------------------------------------------------
  #define DS18B20_STOPINTERRUPTONREAD 0


  // ----------------------------------------------------------------
  //   Prototyp
  // ----------------------------------------------------------------
  int16_t ds18b20_gettemp(void);


  // ----------------------------------------------------------------
  // Praeprozessormacros um 2 Stringtexte zur weiteren Verwendung
  // innerhalb des Praeprozessors  zu verknuepfen
  //
  // Bsp.:
  //        #define ionr      A
  //        #define ioport    CONCAT(PORT, ionr)
  //
  //        ioport wird nun als "PORTA" behandelt

  #define CONC2EXP(a,b)     a ## b
  #define CONCAT(a,b)       CONC2EXP(a, b)
  // ----------------------------------------------------------------

  #define DS18B20_PORT      CONCAT(PORT, ONEWIRE_PORT)
  #define DS18B20_DDR       CONCAT(DDR, ONEWIRE_PORT)
  #define DS18B20_PIN       CONCAT(PIN, ONEWIRE_PORT)
  #define DS18B20_PORTMASK  (1 << ONEWIRE_CONNECT)

  #define onewire_clr()     { DS18B20_PORT &= ~DS18B20_PORTMASK; DS18B20_DDR |= DS18B20_PORTMASK; }
  #define onewire_set()     { DS18B20_DDR &= ~DS18B20_PORTMASK; }
  #define onewire_readb()   (DS18B20_PIN & DS18B20_PORTMASK)

#endif
