/* ----------------------------------------------------------
                         concat_demo.c

   Demonstriert das Verknuepfen innerhalb des Praeprozessors
   von 2 Strings, die weiterverarbeitet werden koennen anhand
   eines einfachen Blinkprogramms.

   Sinn des ganzen ist, bspw. Bezeichnungen von Portpins zu
   vereinfachen, auch wenn die "Aktion im Praeprozessor"
   etwas wirr ist.

     MCU     : alle AVR

   14.03.2024 R. Seelig
   ---------------------------------------------------------- */


#include <util/delay.h>
#include <avr/io.h>

#include "avr_gpio.h"


// LED Anschluss an B5 (Onboard-LED Arduino)

#define led_port      B
#define led_portnum   5

// Inputpin zur Geschwindigkeitsumschaltung

#define key_port      C
#define key_portnum   2


  // ----------------------------------------------------------------
  // Praeprozessormacros um 2 Stringtexte zur weiteren Verwendung
  // innerhalb des Praeprozessors  zu verknuepfen
  //
  // Bsp.:
  //        #define ionr      A
  //        #define ioport    CONCAT(PORT, ionr)
  //
  //        ioport wird nun als "PORTA" behandelt

  #define CONC2EXP(a,b)     a ## b
  #define CONCAT(a,b)       CONC2EXP(a, b)
  // ----------------------------------------------------------------

  // ----------------------------------------------------------------
  // Verknuepfung der Angaben zu led_port und led_portnum zu ent-
  // sprechenden PORTB / DDRB Angaben.
  //
  // Kommentare sind fuer LED an PB5, Eingangspin an C1
  // ----------------------------------------------------------------

  #define LEDPORT           CONCAT(PORT, led_port)    // LEDPORT = PORTB
  #define LEDDDR            CONCAT(DDR, led_port)     // LEDDDR  = DDRB

  #define led_mask          (1 << led_portnum)        // ( 1 << 5 )
  #define led_init()        LEDDDR |= led_mask        // DDRB = ( 1 << 5 )
  #define led_set()         LEDPORT |= led_mask       // PORTB = ( 1 << 5 )
  #define led_clr()         LEDPORT &= ~led_mask      // PORTB &= ~( 1 << 5 )

  #define KEYPORT           CONCAT(PORT, key_port)    // KEYPORT = PORTC
  #define KEYPIN            CONCAT(PIN, key_port)     // KEYPIN  = PINC
  #define KEYDDR            CONCAT(DDR, key_port)     // KEYDDR  = DDRC

  #define key_mask          ( 1 << key_portnum)       // ( 1 << 1 )
  #define key_init()        { KEYDDR &= ~key_mask; KEYPORT |= key_mask; }     // DDRC &= ~ ( 1 << 1 ); PORTC |= ( 1 << 1);
  #define is_key()          ( (KEYPIN & key_mask) >> key_portnum)

#define delay          _delay_ms

#define blkspeed1      500
#define blkspeed2      100

int main(void)
{

  led_init();
  key_init();

  while(1)
  {
    led_set();
    if (is_key()) { delay(blkspeed1); }
             else { delay(blkspeed2); }

    led_clr();
    if (is_key()) { delay(blkspeed1); }
             else { delay(blkspeed2); }
  }
}

