/* -------------------------------------------------------
                         glcd_demo

     Demoprogramm fuer GLCD mit Aufloesung 128 x 128 Pixel

     MCU   :  ATmega328p
     Takt  :  interner oder externer Takt

     21.01.2020  R. Seelig
   ------------------------------------------------------ */

/* -------------------------------------------------------
     In tftdisplay.h wird angegeben, wie das Display
     angeschlossen werden muss..
     Aktuell ist dieses:

                     #define pindefs     1

     in tft_pindefs.h wird dieses ausgewertet und fuer
     die Pindefinition 1 gilt folgende Anschlussbelegung:

     Die Anschlussnamen der Displays variieren fuer ein
     und dieselbe Bedeutung:
                     SDI - SDA
                     CLK - SCK
                     CS  - CE
                     A0  - DC - D/C - RS
                     rst - reset

     Display    Arduino    Atmega    Pinnr.(28 pol.)
     -----------------------------------------------
       sck        D13       PB5          19
       sda        D11       PB3          17
       cs         D10       PB2          16
       DC         D9        PB1          15
       rst        D8        PB0          14

   ------------------------------------------------------- */

#include <util/delay.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <inttypes.h>

#include "tftdisplay.h"
#include "my_printf.h"
#include "gfx_pictures.h"

// Einbinden von Grafiken und Paletten
#include "./include_bmp/iconpal.h"
#include "./include_bmp/icon04.h"
#include "./include_bmp/swgirl01.h"

#include "./include_bmp/manga.h"
#include "./include_bmp/mangapal.h"


#define printf           tiny_printf
#define rgbfrompal(nr)   (pgm_read_word(&(icon04pal[nr])))
#define delay            _delay_ms


/* --------------------------------------------------------
   my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  lcd_putchar(ch);
}

/* -------------------------------------------------------
                          M-A-I-N
   ------------------------------------------------------- */
int main(void)
{
  uint8_t  y,x;
  uint16_t i;
  uint8_t  yofs;

  lcd_init();
  lcd_enable();

  lcd_orientation(3);           // Ausgabeposition auf dem Display

  while(1)
  {

    // ASCII Zeichen ausgeben
    textcolor= rgbfromega(14);
    bkcolor= rgbfromvalue(0x00,0x00,0x66);
    clrscr();
    gotoxy(2,1); printf("    ATmega\n\r");
    gotoxy(2,2); printf("    16MHz\n\r");
    gotoxy(2,3); printf("-------------\n\r");
    gotoxy(2,4); printf("Ascii-Zeichen\n\r");

    for (x= 32; x< 128; x++)
    {
      if ((x % 16)== 0) printf("\n\r");
      my_putchar(x);
    }
    delay(5000);

    // Demo, fuer putpixel, Linien, Kreise
    textcolor= rgbfromega(14);
    bkcolor= 0;                         // schwarz
    clrscr();
    bkcolor= rgbfromvalue(0x20,0x20,0xff);
    fillrect(6,6,122,122,rgbfromvalue(0x20,0x20,0xff));
    gotoxy(2,1); printf("    ATmega\n\r");
    gotoxy(2,2); printf("    16MHz\n\r");
    gotoxy(2,3); printf("-------------\n\r");
    gotoxy(2,4); printf("by R.Seelig");

    fillcircle(64,85,25, rgbfromvalue(0x66,0x00,0x00));
    for (x= 51; x< 121; x++)
    {
      putpixel(x,x,rgbfromvalue(0x00,0x80,0x00));
    }
    line(30,51,97,121,rgbfromega(yellow));

    delay(2500);

    bkcolor= rgbfromvalue(0xa8, 0x79, 0x1c);
    clrscr();

    // sw-Pixelgrafik aus tftdisplay.c
    showimage(0,0,&girl01bmp[0],rgbfromega(0));
    textcolor= rgbfromega(15);
    gotoxy(2,14); printf(" Pixelgrafik ");
    delay(3000);

    // Linien ueber die Pixelgrafik legen
    clrscr();
    showimage(0,0,&girl01bmp[0],rgbfromega(0));

    for (x= 0; x< 125; x+= 5)
    {
      line(0, 127, x, 0, rgbfromega(yellow));
      delay(50);
    }

    for (x= 0; x< 125; x+= 5)
    {
      line(127, 127, x, 0, rgbfromega(lightred));
      delay(50);
    }
    delay(2000);

    bkcolor= rgbfromega(blue);
    textcolor= rgbfromega(yellow);
    clrscr();
    fillrect(0,0,127,23,rgbfromega(1));
    gotoxy(0,1); printf("  Palettentest");

    y= 2;
    gotoxy(0,y);
    for (i= 0; i< (256-48); i++)
    {
      if ((i % 16)== 0)
      {
        y++;
        gotoxy(0,y);
      }
      bkcolor= rgbfrompal(i);
      my_putchar(' ');
    }
    delay(2000);

    bkcolor= rgbfrompal(i);
    clrscr();
    // Bitmap anzeigen mit Code in gfx_pictures.c
    bmp256_show(0,0,&mangabmp[0], &mangapal[0]);

    textcolor= rgbfromega(4);
    bkcolor= rgbfromega(15);
    gotoxy(3,1); printf("Bitmap aus");
    gotoxy(5,2); printf("image2c");

    delay(4000);

    bkcolor= rgbfrompal(255);
    clrscr();

    textcolor= rgbfromega(1);
    gotoxy(2,13); printf("  Animation");
    gotoxy(2,14); printf("mit ATmega328");
    yofs= 25;
    for (x= 0; x < (127-33); x+= 1)
    {
      bmp256_show(x,yofs,&bmpimage04[0], &icon04pal[0]);
      delay(20);
    }
    for (x = (127-33); x > 0; x-= 1)
    {
      bmp256_show(x,yofs,&bmpimage04[0], &icon04pal[0]);
      delay(20);
    }
  }
}
