/* -----------------------------------------------------------------------------------
                                     tft_pindefs.h

     Pinbelegungen TFT-Display zu Controller

     MCU : ATmega


     07.05.2019  R. Seelig
   ----------------------------------------------------------------------------------- */

#ifndef in_tft_pindefs
  #define in_tft_pindefs

  #include "tftdisplay.h"
  #include "avr_gpio.h"

  #if (USE_SPI_TFT == 1)

    /*  ------------------------------------------------------------
                           Pinbelegung SPI

          hier koennen unterschiedliche Pinbelegungen fuer die An-
          schluesse rst, dc und ce angegeben werden. Da Hardware-SPI
          verwedet wird, sind die Anschluesse fuer MOSI, CLK und SS
          fest mit PB3, PB5 und PB2 vorgegeben.

          In tftdisplay.h wird der Wert fuer pindefs vorgegeben.
          Das ermoeglicht hier verschiedene Pinbelegungen abzulegen
        ------------------------------------------------------------ */

    #if (pindefs == 2)

      // Anschluss reset
      #define _lcd_rst_port         D
      #define lcd_rst_pin           6

      // Anschluss dc
      #define _lcd_dc_port          D
      #define lcd_dc_pin            7

      // Anschluss ce
      #define _lcd_ce_port          B
      #define lcd_ce_pin            2

    #endif

  #endif

  // ----------------------------------------------------------------
  // Praeprozessormacros um 2 Stringtexte zur weiteren Verwendung
  // innerhalb des Praeprozessors  zu verknuepfen
  //
  // Bsp.:
  //        #define ionr      A
  //        #define ioport    CONCAT(PORT, ionr)
  //
  //        ioport wird nun als "PORTA" behandelt

  #define CONC2EXP(a,b)     a ## b
  #define CONCAT(a,b)       CONC2EXP(a, b)
  // ----------------------------------------------------------------


  #if (USE_SPI_TFT == 1)


    // da Hardware-SPI verwendet wird, sind die Anschluesse fuer MOSI, CLK und SS
    // fest vorgegeben (fuer alle evtl. Pinbelegungen)
    #define spi_ddr                 DDRB
    #define spi_mosi_pin            PB3          // Arduino digital 11
    #define spi_clk_pin             PB5          //      "          13
    #define spi_ss_pin              PB2          //      "          10

    #define rst_mask                (1 << lcd_rst_pin)
    #define lcd_rst_port            CONCAT(PORT, _lcd_rst_port)
    #define lcd_rst_ddr             CONCAT(DDR, _lcd_rst_port)

    #define dc_mask                (1 << lcd_dc_pin)
    #define lcd_dc_port            CONCAT(PORT, _lcd_dc_port)
    #define lcd_dc_ddr             CONCAT(DDR, _lcd_dc_port)

    #define ce_mask                (1 << lcd_ce_pin)
    #define lcd_ce_port            CONCAT(PORT, _lcd_ce_port)
    #define lcd_ce_ddr             CONCAT(DDR, _lcd_ce_port)

    #define rst_set()               ( lcd_rst_port |= rst_mask )
    #define rst_clr()               ( lcd_rst_port &= ~rst_mask )
    #define lcd_rst_init()          ( lcd_rst_ddr |= rst_mask )

    #define dc_set()                ( lcd_dc_port |= (1 << lcd_dc_pin) )
    #define dc_clr()                ( lcd_dc_port &= ~(1 << lcd_dc_pin) )
    #define lcd_dc_init()           ( lcd_dc_ddr |= (1 << lcd_dc_pin) )

    #define ce_set()                ( lcd_ce_port |= (1 << lcd_ce_pin) )
    #define ce_clr()                ( lcd_ce_port &= ~(1 << lcd_ce_pin) )
    #define lcd_ce_init()           ( lcd_ce_ddr |= (1 << lcd_ce_pin) )

    #define lcd_pin_init()          { lcd_rst_init(); lcd_dc_init(); lcd_ce_init(); }
    #define lcd_enable()            ( ce_clr() )
    #define lcd_disable()           ( ce_set() )


  #endif               // USE_SPI_TFT


  #if (USE_8BIT_TFT == 1)
    /*  ------------------------------------------------------------
          Pinbelegung fuer TFT-Displays mit 8-Bit Parallelinterface
        ------------------------------------------------------------ */

    #define intbit(wert,nr)    ((wert) & (1<<nr))
    #define setintbit(wert,nr) (wert |= (1<<nr))
    #define clrintbit(wert,nr) (wert &= ~(1<<nr))

    //  -------------------------------------------------------------
    //
    //  -------------------------------------------------------------
    #if (boardversion == 0)                                       //

      #define lcd_d0_init()   PB0_output_init()
      #define lcd_d0_set()    PB0_set()
      #define lcd_d0_clr()    PB0_clr()

      #define lcd_d1_init()   PB1_output_init()
      #define lcd_d1_set()    PB1_set()
      #define lcd_d1_clr()    PB1_clr()

      #define lcd_d2_init()   PD2_output_init()
      #define lcd_d2_set()    PD2_set()
      #define lcd_d2_clr()    PD2_clr()

      #define lcd_d3_init()   PD3_output_init()
      #define lcd_d3_set()    PD3_set()
      #define lcd_d3_clr()    PD3_clr()

      #define lcd_d4_init()   PD4_output_init()
      #define lcd_d4_set()    PD4_set()
      #define lcd_d4_clr()    PD4_clr()

      #define lcd_d5_init()   PD5_output_init()
      #define lcd_d5_set()    PD5_set()
      #define lcd_d5_clr()    PD5_clr()

      #define lcd_d6_init()   PD6_output_init()
      #define lcd_d6_set()    PD6_set()
      #define lcd_d6_clr()    PD6_clr()

      #define lcd_d7_init()   PD7_output_init()
      #define lcd_d7_set()    PD7_set()
      #define lcd_d7_clr()    PD7_clr()

      #define lcd_rd_init()   PC0_input_init()
      #define lcd_rd_set()    PC0_set()
      #define lcd_rd_clr()    PC0_clr()

      #define lcd_wr_init()   PC1_output_init()
      #define lcd_wr_set()    PC1_set()
      #define lcd_wr_clr()    PC1_clr()

      #define lcd_rs_init()   PC2_output_init()
      #define lcd_rs_set()    PC2_set()
      #define lcd_rs_clr()    PC2_clr()

      #define lcd_cs_init()   PC3_output_init()
      #define lcd_cs_set()    PC3_set()
      #define lcd_cs_clr()    PC3_clr()

      #define lcd_rst_init()   PC4_output_init()
      #define lcd_rst_set()    PC4_set()
      #define lcd_rst_clr()    PC4_clr()


    #endif

    //  -------------------------------------------------------------
    //
    //  -------------------------------------------------------------
    #if (boardversion == 1)


    #endif

  #endif        // USE_8BIT_TFT

#endif
