/* ----------------------------------------------------------
     smallio.h

     Header fuer ein Sehr schlankes I/O Interface ueber die
     serielle Schnittstelle mit abgespecktem printf. Hier-
     durch ist es moeglich, die Beispiele der Hilfedatei
     fuer viele Mikrocontroller und auf PC nachzuvollziehen.

     Fuer die unterstuetzten Microcontroller (AVR, STM32,
     MCS-51, STM8) ist jeweils ein eigenes smallio
     verfuegbar, sodass JEDES Programm lediglich im Header
     ein

                  "include smallio.h"

     eingefuegt werden muss. Im Makefile ist einzustellen,
     dass smallio.o fuer AVR,ARM und PC-Konsolenprogramme
     oder smallio.rel fuer  MCS-51 und STM8) hinzugelinkt
     werden muss. Eintrag im Makefile:

     SRCS      += ../src/smallio.o

     Die eingestellte Baudrate betraegt bei allen
     Controllern 19200 bd (Ausnahme MCS-51, hier dann
     4800 bd)

     Protokoll ist 8N1

     ------------------------------------------------------

     Von smallio Unterstuetzte Controller in der
     AVR-Version:

        Controller mit Hardware-UART

        - ATmega8
        - ATmega8515
        - ATmega48 - ATmega328
        - ATtiny2313, ATtiny4313

        Controller mit USI-Serial

        - ATtiny24 - ATtiny84
        - ATtiny25 - ATtiny85


        Anschlussbelegungen:

                MCU            PORT    Funk.1   Funk.2
        ---------------------------------------------------
        ATtiny25 - ATtiny85    PB1      TxD       Do   (6)
                               PB0      RxD       Di   (5)
        ---------------------------------------------------
        ATtiny24 - ATtiny84    PA5      TxD       Do   (8)
                               PA6      RxD       Di   (7)
        ---------------------------------------------------
        ATtiny2313 / 4313      PD1      TxD            (3)
                               PD0      RxD            (2)
        ---------------------------------------------------
        28 pol. ATmega (8, 48, 88, 168, 328)
        ---------------------------------------------------
                               PD1      TxD            (3)
                               PD0      RxD            (2)
        ---------------------------------------------------

     18.6.2019  R. Seelig

   ---------------------------------------------------------- */

#ifndef in_small_io
  #define in_small_io

  #include <stdint.h>
  #include <stdlib.h>
  #include <stdbool.h>
  #include <stdarg.h>


  #include <util/delay.h>
  #include <avr/io.h>
  #include <avr/pgmspace.h>
  #include <avr/interrupt.h>

  #include "avr_gpio.h"


  /* -----------------------------------------------------------------------
      Moegliche Baudratenkombinationen fuer ATtiny24 - 84, ATtiny25 - 85

      F_CPU 1000000   BAUDRATE 1200, 2400
      F_CPU 8000000   BAUDRATE 4800, 9600, 19200
      F_CPU 12000000  BAUDRATE 4800, 9600, 19200,
      F_CPU 16000000  BAUDRATE 4800, 9600, 19200, 38400

      Moegliche Baudraten fuer ATmega / ATtiny2313

      mit 16 MHz Quarz bis 115200
     ----------------------------------------------------------------------- */

  #undef putchar
  #define BAUDRATE        19200

  /* -----------------------------------------------------------------------
                                  Prototypen
     ----------------------------------------------------------------------- */
  void    smallio_init(void);
  int     putchar(int ch);
  uint8_t keypressed( void );
  uint8_t getchar( void );
  int16_t readint();

  void    putint(int16_t i, char komma);
  void    hexnibbleout(uint8_t b);
  void    puthex(uint16_t h);
  void    my_putramstring(uint8_t *p);
  void    my_printf(const uint8_t *s,...);

  #define printf(str,...)  (my_printf(PSTR(str), ## __VA_ARGS__))
  #define delay            _delay_ms


  char printfkomma;

#endif
