/* -----------------------------------------------------------------------------------
                                     tft_pindefs.h

     Pinbelegungen TFT-Display zu Controller

     MCU : ATmega


     07.05.2019  R. Seelig
   ----------------------------------------------------------------------------------- */

#ifndef in_tft_pindefs
  #define in_tft_pindefs

  #include "tftdisplay.h"
  #include "avr_gpio.h"

  #if (USE_SPI_TFT == 1)

  /*  ------------------------------------------------------------
                         Pinbelegung SPI
      ------------------------------------------------------------ */
  #if (pindefs == 1)
                                                 // Arduino digital     28 pol. Chip
    #define spi_ddr                 DDRB
    #define spi_mosi_pin            PB3          //     11                  17
    #define spi_clk_pin             PB5          //     13                  19
    #define spi_ss_pin              PB2          //     10                  16

    // Anschluss reset
    #define lcd_rst_port            PORTB
    #define lcd_rst_ddr             DDRB
    #define lcd_rst_pin             0            //      8                  14

   // Anschluss dc
    #define lcd_dc_port             PORTB
    #define lcd_dc_ddr              DDRB
    #define lcd_dc_pin              1           //       9                  15

    // Anschluss ce
    #define lcd_ce_port             PORTB
    #define lcd_ce_ddr              DDRB
    #define lcd_ce_pin              2           // gleicher Anschluss spi_ss


    #define rst_set()               ( lcd_rst_port |= (1 << lcd_rst_pin) )
    #define rst_clr()               ( lcd_rst_port &= ~(1 << lcd_rst_pin) )
    #define lcd_rst_init()          ( lcd_rst_ddr |= (1 << lcd_rst_pin) )

    #define dc_set()                ( lcd_dc_port |= (1 << lcd_dc_pin) )
    #define dc_clr()                ( lcd_dc_port &= ~(1 << lcd_dc_pin) )
    #define lcd_dc_init()           ( lcd_dc_ddr |= (1 << lcd_dc_pin) )

    #define ce_set()                ( lcd_ce_port |= (1 << lcd_ce_pin) )
    #define ce_clr()                ( lcd_ce_port &= ~(1 << lcd_ce_pin) )
    #define lcd_ce_init()           ( lcd_ce_ddr |= (1 << lcd_ce_pin) )

    #define lcd_pin_init()          { lcd_rst_init(); lcd_dc_init(); lcd_ce_init(); }
    #define lcd_enable()            ( ce_clr() )
    #define lcd_disable()           ( ce_set() )

  #endif


  #if (pindefs == 2)

    #define spi_ddr                 DDRB
    #define spi_mosi_pin            PB3          // Arduino digital 11
    #define spi_clk_pin             PB5          //      "          13
    #define spi_ss_pin              PB2          //      "          10

    // Anschluss reset
    #define lcd_rst_port            PORTD
    #define lcd_rst_ddr             DDRD
    #define lcd_rst_pin             6

   // Anschluss dc
    #define lcd_dc_port             PORTD
    #define lcd_dc_ddr              DDRD
    #define lcd_dc_pin              7

    // Anschluss ce
    #define lcd_ce_port             PORTB
    #define lcd_ce_ddr              DDRB
    #define lcd_ce_pin              2


    #define rst_set()               ( lcd_rst_port |= (1 << lcd_rst_pin) )
    #define rst_clr()               ( lcd_rst_port &= ~(1 << lcd_rst_pin) )
    #define lcd_rst_init()          ( lcd_rst_ddr |= (1 << lcd_rst_pin) )

    #define dc_set()                ( lcd_dc_port |= (1 << lcd_dc_pin) )
    #define dc_clr()                ( lcd_dc_port &= ~(1 << lcd_dc_pin) )
    #define lcd_dc_init()           ( lcd_dc_ddr |= (1 << lcd_dc_pin) )

    #define ce_set()                ( lcd_ce_port |= (1 << lcd_ce_pin) )
    #define ce_clr()                ( lcd_ce_port &= ~(1 << lcd_ce_pin) )
    #define lcd_ce_init()           ( lcd_ce_ddr |= (1 << lcd_ce_pin) )

    #define lcd_pin_init()          { lcd_rst_init(); lcd_dc_init(); lcd_ce_init(); }
    #define lcd_enable()            ( ce_clr() )
    #define lcd_disable()           ( ce_set() )

  #endif

  #if (pindefs == 3)

    #define spi_ddr                 DDRB
    #define spi_mosi_pin            PB3          // Arduino digital 11
    #define spi_clk_pin             PB5          //      "          13
    #define spi_ss_pin              PB2          //      "          10

    // Anschluss reset
    #define lcd_rst_port            PORTB
    #define lcd_rst_ddr             DDRB
    #define lcd_rst_pin             1

   // Anschluss dc
    #define lcd_dc_port             PORTB
    #define lcd_dc_ddr              DDRB
    #define lcd_dc_pin              0

    // Anschluss ce
    #define lcd_ce_port             PORTB
    #define lcd_ce_ddr              DDRB
    #define lcd_ce_pin              2


    #define rst_set()               ( lcd_rst_port |= (1 << lcd_rst_pin) )
    #define rst_clr()               ( lcd_rst_port &= ~(1 << lcd_rst_pin) )
    #define lcd_rst_init()          ( lcd_rst_ddr |= (1 << lcd_rst_pin) )

    #define dc_set()                ( lcd_dc_port |= (1 << lcd_dc_pin) )
    #define dc_clr()                ( lcd_dc_port &= ~(1 << lcd_dc_pin) )
    #define lcd_dc_init()           ( lcd_dc_ddr |= (1 << lcd_dc_pin) )

    #define ce_set()                ( lcd_ce_port |= (1 << lcd_ce_pin) )
    #define ce_clr()                ( lcd_ce_port &= ~(1 << lcd_ce_pin) )
    #define lcd_ce_init()           ( lcd_ce_ddr |= (1 << lcd_ce_pin) )

    #define lcd_pin_init()          { lcd_rst_init(); lcd_dc_init(); lcd_ce_init(); }
    #define lcd_enable()            ( ce_clr() )
    #define lcd_disable()           ( ce_set() )
  #endif

  #if (pindefs == 4)
    // Markus Schulz Display

    // Waitstates fuer KMR-1.8 128x160 Displays
    // notwendig, da anscheinend die neueren SPI-Displays auf dem Interface nicht
    // so schnell sind, wie die aelteren

    #define kmr_1_8_ws              2

    #define spi_ddr                 DDRB
    #define spi_mosi_pin            PB3          // Arduino digital 11
    #define spi_clk_pin             PB5          //      "          13
    #define spi_ss_pin              PB2          //      "          10

    // Anschluss reset
    #define lcd_rst_port            PORTB
    #define lcd_rst_ddr             DDRB
    #define lcd_rst_pin             0

   // Anschluss dc
    #define lcd_dc_port             PORTB
    #define lcd_dc_ddr              DDRB
    #define lcd_dc_pin              1

    // Anschluss ce
    #define lcd_ce_port             PORTB
    #define lcd_ce_ddr              DDRB
    #define lcd_ce_pin              2


    #define rst_set()               { (lcd_rst_port |= (1 << lcd_rst_pin)); _delay_us(kmr_1_8_ws); }
    #define rst_clr()               { (lcd_rst_port &= ~(1 << lcd_rst_pin)); _delay_us(kmr_1_8_ws); }
    #define lcd_rst_init()          { (lcd_rst_ddr |= (1 << lcd_rst_pin)); _delay_us(kmr_1_8_ws); }

    #define dc_set()                { (lcd_dc_port |= (1 << lcd_dc_pin)); _delay_us(kmr_1_8_ws); }
    #define dc_clr()                { (lcd_dc_port &= ~(1 << lcd_dc_pin)); _delay_us(kmr_1_8_ws); }
    #define lcd_dc_init()           { (lcd_dc_ddr |= (1 << lcd_dc_pin)); _delay_us(kmr_1_8_ws); }

    #define ce_set()                { (lcd_ce_port |= (1 << lcd_ce_pin)); _delay_us(kmr_1_8_ws); }
    #define ce_clr()                { (lcd_ce_port &= ~(1 << lcd_ce_pin)); _delay_us(kmr_1_8_ws); }
    #define lcd_ce_init()           { (lcd_ce_ddr |= (1 << lcd_ce_pin)); _delay_us(kmr_1_8_ws); }

    #define lcd_pin_init()          { lcd_rst_init(); lcd_dc_init(); lcd_ce_init(); }
    #define lcd_enable()            ( ce_clr() )
    #define lcd_disable()           ( ce_set() )
  #endif


  #endif               // USE_SPI_TFT

  #if (USE_8BIT_TFT == 1)
    /*  ------------------------------------------------------------
          Pinbelegung fuer TFT-Displays mit 8-Bit Parallelinterface
        ------------------------------------------------------------ */

    #define intbit(wert,nr)    ((wert) & (1<<nr))
    #define setintbit(wert,nr) (wert |= (1<<nr))
    #define clrintbit(wert,nr) (wert &= ~(1<<nr))

    //  -------------------------------------------------------------
    //
    //  -------------------------------------------------------------
    #if (boardversion == 0)                                       //

      #define lcd_d0_init()   PB0_output_init()
      #define lcd_d0_set()    PB0_set()
      #define lcd_d0_clr()    PB0_clr()

      #define lcd_d1_init()   PB1_output_init()
      #define lcd_d1_set()    PB1_set()
      #define lcd_d1_clr()    PB1_clr()

      #define lcd_d2_init()   PD2_output_init()
      #define lcd_d2_set()    PD2_set()
      #define lcd_d2_clr()    PD2_clr()

      #define lcd_d3_init()   PD3_output_init()
      #define lcd_d3_set()    PD3_set()
      #define lcd_d3_clr()    PD3_clr()

      #define lcd_d4_init()   PD4_output_init()
      #define lcd_d4_set()    PD4_set()
      #define lcd_d4_clr()    PD4_clr()

      #define lcd_d5_init()   PD5_output_init()
      #define lcd_d5_set()    PD5_set()
      #define lcd_d5_clr()    PD5_clr()

      #define lcd_d6_init()   PD6_output_init()
      #define lcd_d6_set()    PD6_set()
      #define lcd_d6_clr()    PD6_clr()

      #define lcd_d7_init()   PD7_output_init()
      #define lcd_d7_set()    PD7_set()
      #define lcd_d7_clr()    PD7_clr()

      #define lcd_rd_init()   PC0_input_init()
      #define lcd_rd_set()    PC0_set()
      #define lcd_rd_clr()    PC0_clr()

      #define lcd_wr_init()   PC1_output_init()
      #define lcd_wr_set()    PC1_set()
      #define lcd_wr_clr()    PC1_clr()

      #define lcd_rs_init()   PC2_output_init()
      #define lcd_rs_set()    PC2_set()
      #define lcd_rs_clr()    PC2_clr()

      #define lcd_cs_init()   PC3_output_init()
      #define lcd_cs_set()    PC3_set()
      #define lcd_cs_clr()    PC3_clr()

      #define lcd_rst_init()   PC4_output_init()
      #define lcd_rst_set()    PC4_set()
      #define lcd_rst_clr()    PC4_clr()


    #endif

    //  -------------------------------------------------------------
    //
    //  -------------------------------------------------------------
    #if (boardversion == 1)


    #endif

  #endif        // USE_8BIT_TFT

#endif
