/* ----------------------------------------------------------
                          ws2812.h

     Softwaremodul zum Ansteuern von (kaskadierten) WS2812
     Leuchtdioden

     MCU      :  AVR
     Takt     :  8 MHz oder 16 MHz
                 die Dateien
                     - ws2812_output_16mhz.S
                     - WS2812_output_8mhz.S
                 legen fest, fuer welche Taktgeschwindigkeit
                 die Funktionen ausgelegt sind. Eine der
                 beiden Dateien muss zum Main-Programm hinzu-
                 gelinkt werden

     Anschlusspins siehe ws2812_pins.h

     05.02.2020  R. Seelig
   ---------------------------------------------------------- */


#ifndef in_ws2812
  #define in_ws2812

  #include <util/delay.h>
  #include <avr/io.h>
  #include <avr/interrupt.h>
  #include <avr/pgmspace.h>
  #include <string.h>

  #include "ws2812_pins.h"

  // Farbstruktur rot, gruen, blau
  typedef struct colvalue
  {
    uint8_t r;
    uint8_t g;
    uint8_t b;
  } colvalue;

  #define ws_black      0
  #define ws_bluedrk    1
  #define ws_greendrk   2
  #define ws_cyandrk    3
  #define ws_reddrk     4
  #define ws_purpledrk  5
  #define ws_brown      6
  #define ws_grey       7
  #define ws_greydrk    8
  #define ws_blue       9
  #define ws_green      10
  #define ws_cyan       11
  #define ws_red        12
  #define ws_purple     13
  #define ws_yellow     14
  #define ws_white      15

  extern  const uint8_t PROGMEM egapalette[];

  /* ----------------------------------------------------------
                            Prototypen
     ---------------------------------------------------------- */
  void rgbfromega(uint8_t eganr, struct colvalue *f);
  void rgbfromvalue(uint8_t r, uint8_t g, uint8_t b, struct colvalue *f);
  void ws_showbuffer(uint8_t *ptr, uint16_t count);
  void ws_setrgbcol(uint8_t *ptr, uint16_t nr, struct colvalue *f);
  void ws_reset(void);
  void ws_init(void);
  void ws_clrarray(uint8_t *ptr, int anz);
  void ws_blendup_left(uint8_t *ptr, uint8_t anz, struct colvalue *f, int dtime);
  void ws_blendup_right(uint8_t *ptr, uint8_t anz, struct colvalue *f, int dtime);
  void ws_buffer_rl(uint8_t *ptr, uint8_t lanz);
  void ws_buffer_rr(uint8_t *ptr, uint8_t lanz);

#endif
