/*   -------------------------------------------------------
                          hd44780.h

     Header fuer grundlegende Funktionen eines HD44780
     kompatiblen LC-Textdisplays

     Hardware : Text-LCD

     MCU      :  AVR
     Takt     :

     Fuses    :

     17.09.2018  R. Seelig

   ------------------------------------------------------- */

/*
      Bsp.-Anschluss am Bsp. Pollin-Display C0802-04 zu
      einem ATmega
      Pinkonfiguration fuer - lcdpindef1 -
      ---------------------------------------------------
         o +5V
         |                            Display                 Controller
         _                        Funktion   PIN            PIN    Funktion
        | |
        |_|                          GND      1 ------------
         |                          +5V       2 ------------
         o----o Kontrast   ---    Kontrast    3 ------------
        _|_                           RS      4 ------------   11    PD5
        \ /                          GND      5 ------------
        -'-                    (Takt) E       6 ------------   12    PD6
         |                           D4       7 ------------   13    PD7
        --- GND                      D5       8 ------------   14    PB0
                                     D6       9 ------------   15    PB1
                                     D7      10 ------------   16    PB2
*/

#ifndef in_txlcd_5v
  #define in_txlcd_5v

  #include <util/delay.h>
  #include <avr/io.h>
  #include <avr/pgmspace.h>
  #include <stdint.h>

  #include "avr_gpio.h"


  /* -------------------------------------------------------
        Pinbelegung
     ------------------------------------------------------- */

  #define rs_init()      PD5_output_init()           // command / data
  #define e_init()       PD6_output_init()           // clock

  #define d4_init()      PD7_output_init()
  #define d5_init()      PB0_output_init()
  #define d6_init()      PB1_output_init()
  #define d7_init()      PB2_output_init()

  #define rs_set()       PD5_set()
  #define rs_clr()       PD5_clr()

  #define e_set()        PD6_set()
  #define e_clr()        PD6_clr()

  #define d4_set()       PD7_set()
  #define d4_clr()       PD7_clr()

  #define d5_set()       PB0_set()
  #define d5_clr()       PB0_clr()

  #define d6_set()       PB1_set()
  #define d6_clr()       PB1_clr()

  #define d7_set()       PB2_set()
  #define d7_clr()       PB2_clr()


  #define txlcd_prints(tx)      (txlcd_putromstring(PSTR(tx)))       // Anzeige String aus Flashrom: prints("Text");

  /* -------------------------------------------------------
       diverse Macros
     ------------------------------------------------------- */

  #define testbit(reg,pos) ((reg) & (1<<pos))               // testet an der Bitposition pos das Bit auf 1 oder 0


  /* -------------------------------------------------------
       Prototypen
     ------------------------------------------------------- */

    void txlcd_init(void);
    void txlcd_setuserchar(uint8_t nr, const uint8_t *userchar);
    void gotoxy(uint8_t x, uint8_t y);
    void txlcd_putchar(char ch);
    void txlcd_putramstring(uint8_t *c);
    void txlcd_putromstring(const uint8_t *dataPtr);

    #define clrscr()      txlcd_init()


    extern uint8_t wherex,wherey;

#endif
