/* -------------------------------------------------------
                      txtlcd_demo.c

     Testprogramm fuer ein Standardtextdisplay mit
     HD44780 kompatiblen LC-Controller, 4-Bit Daten-
     transfermodus

     Hardware : Text-LCD

     MCU      :  AVR
     Takt     :

     13.02.2020  R. Seelig

   ------------------------------------------------------- */

/*
      Default Displayanschluss an den Controller
      ----------------------------------------------------------
         o +5V
         |                            Display                    AVR
         _                        Funktion   PIN             PIN   Funktion
        | |                                                 (ATmega 28 pol.)
        |_|                          GND      1 ------------
         |                          +5V       2 ------------
         o----o Kontrast   ---    Kontrast    3 ------------
        _|_                           RS      4 ------------  19     PB5
        \ /                        rd /wr     5 ------------  GND
        -'-                    (Takt) E       6 ------------  18     PB4
         |                           D0       7                      n.c.
        ---                          D1       8                      n.c.
                                     D2       9                      n.c.
                                     D3      10                      n.c.
                                     D4      11 ------------  26     PC3
                                     D5      12 ------------  17     PB3
                                     D6      13 ------------  27     PC4
                                     D7      14 ------------  16     PB2


                                            +-------------+
                    PC6 - /reset - PCINT14  |  1       28 |  PC5 - ADC5 - SCL - PCINT13 (A5)
                  (D0) PD0 - PCINT16 - RxD  |  2       27 |  PC4 - ADC4 - SDA - PCINT12 (A4)
                  (D1) PD1 - PCINT17 - TxD  |  3   A   26 |  PC3 - ADC3 - PCINT11 (A3)
                 (D2) PD2 - PCINT18 - INT0  |  4   T   25 |  PC2 - ADC2 - PCINT10 (A2)
          (D3) PD3 - PCINT19 - OC2B - INT1  |  5   m   24 |  PC1 - ADC1 - PCINT9 (A1)
             (D4) PD4 - PCINT20 - XCK - T0  |  6   e   23 |  PC0 - ADC0 - PCINT8 (A0)
                                       Vcc  |  7   g   22 |  Gnd
                                       Gnd  |  8   a   21 |  Aref
              PB6 - PCINT6 - XTAL1 - TOSC1  |  9       20 |  AVcc
              PB7 - PCINT7 - XTAL2 - TOSC2  | 10       19 |  PB5 - SCK - PCINT5 (D13)
           (D5) PD5 - PCINT21 -  OC0B - T1  | 11       18 |  PB4 - MISO - PCINT4 (D12)
         (D6) PD6 - PCINT22 -  OC0A - AIN0  | 12       17 |  PB3 - MOSI - OC2A - PCINT13 (D11 - PWM)
                 (D7) PD7 - PCINT23 - AIN1  | 13       16 |  PB2 - SS - OC1B - PCINT2 (D10 - PWM)
           (D8) PB0 - PCINT0 - CLKO - ICP1  | 14       15 |  PB1 - OC1A - PCINT1 (D9 - PWM)
                                            +-------------+
*/

// ------------------------------------------------------------------------------------------------------------------


#include <util/delay.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <stdint.h>

#include "hd44780.h"
#include "my_printf.h"

#define printf    my_printf
#define delay     _delay_ms

static const unsigned char charbmp0[8]  =         // hochgestelltes "o" (fuer Gradangaben)
  { 0x0c, 0x12, 0x12, 0x0c, 0x00, 0x00, 0x00, 0x00 };

static const unsigned char charbmp1[8]  =         // Pfeil nach links
  { 0x08, 0x0c, 0x0e, 0x0f, 0x0e, 0x0c, 0x08, 0x00 };

static const unsigned char ohmbmp[8]    =           // Ohmzeichen
  { 0x0e, 0x11, 0x11, 0x11, 0x0a, 0x0a, 0x1b, 0x00};

/* --------------------------------------------------------
   my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  txlcd_putchar(ch);
}


/* -------------------------------------------------------
                          M-A-I-N
   ------------------------------------------------------- */
int main()
{
  uint16_t cx= 0;


  txlcd_init();

  txlcd_setuserchar(0,&charbmp0[0]);
  txlcd_setuserchar(1,&charbmp1[0]);
  txlcd_setuserchar(2,&ohmbmp[0]);

  gotoxy(1,1); printf("UserChar");
  gotoxy(1,2); printf("%c %c %c", 0,1,2);
  delay(1000);
  clrscr();
  gotoxy(1,1); printf("Counter");
  while(1)
  {
    gotoxy(1,2); printf("        ");
    gotoxy(1,2); printf("cx= %d",cx);
//    gotoxy(1,2); printf("%c",'8');
    cx++;
    cx = cx % 1000;
    delay(500);
  }
}
