/* ----------------------------------------------------------
     millis.c

     Timerinterrupt zaehlt die long int Variable millis hoch
     (und stellt somit ein Arduino Pendant her)

     MCU     : atmega328p
     F_CPU   : 16000000

     14.2.2018

   ---------------------------------------------------------- */

#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "avr_gpio.h"


#if defined __AVR_ATmega8__    || __AVR_ATmega8515__  ||                                            \
            __AVR_ATtiny2313__ || __AVR_ATtiny4313__

  #define TIMSK1   TIMSK

#endif


#define led_init()      PB5_output_init()
#define led_set()       PB5_set()
#define led_clr()       PB5_clr()

volatile uint32_t millis = 0;

ISR (TIMER1_COMPA_vect)
{
  millis++;
}

void timer1_init(void)
{
  /* ---------------------------------------------
     CS10 / CS11 / CS12 sind Steuerbits
     des Taktvorteilers.

     CS 12 | CS11 | CS10
     ----------------------------------------------
       0      0      0   kein Takt, Timer gestoppt
       0      0      1   clk / 1 (kein Teiler)
       0      1      0   clk / 8
       0      1      1   clk / 64
       1      0      0   clk / 256
       1      0      1   clk / 1024
     ---------------------------------------------- */
  TCCR1B = 1 << WGM12 | 1 << CS10;
  OCR1A = F_CPU / 1000;                 // 1000 = Reloadwert fuer 16 MHz
  TCNT1 = 0;

  TIMSK1 = 1 << OCIE1A;
  sei();
}

void tim1_delay(uint32_t dtime)
{
  volatile uint32_t now;

  now= millis;

  while(now + dtime > millis);
}

/* ---------------------------------------------------------------------------
                                    M A I N
   --------------------------------------------------------------------------- */
int main(void)
{
  // Mainprogram starts here
  timer1_init();
  led_init();

  while(1)
  {
    tim1_delay(500);
    led_set();
    tim1_delay(500);
    led_clr();
  }
}
