/* ---------------------------------------------------------------
                         my_printf32_demo.c

     Demoprogramm fuer my_printf / my_sprintf mit Ausgabe ueber
     eine serielle Schnittstelle

     MCU : AVR-Mikrocontroller mit Flashspeicher > 2kByte
     Takt: beliebig

     29.03.2024   R. Seelig
   -------------------------------------------------------------- */

#include <avr/io.h>
#include <string.h>

#include "my_printf32.h"
#include "uart_all.h"


#define printf    my_printf             // printf auf eigene Funktionen "umleiten"
#define sprintf   my_sprintf            // dto. fuer sprintf

#define delay     _delay_ms


/* --------------------------------------------------------
   my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  uart_putchar(ch);
}

/* --------------------------------------------------
                       putstr

     gibt einen Ascii-Z String aus
   -------------------------------------------------- */
void putstr(char *p)
{
  while(*p)
  {
    my_putchar( *p );
    p++;
  }
}


/* --------------------------------------------------------
                             main
   -------------------------------------------------------- */
int main(void)
{
  char    txtbuffer[50];
  uint8_t counter;

  uint8_t z;
  float a, b, c;

  uint32_t a2;

  uart_init();

  printf("\n\n\rDemo fuer 'my_printf32.c'");
  printf("  \n\r-------------------------------------\n\r");

  a= 42.16; b= 13.24;
  c= a*b;
  a2= c*10000;                      // c um 4 dezimale Stellen nach links verschieben
                                    // um dann mittels %k Platzhalter 4 Nachkommastellen
                                    // anzeigen zu koennen
  printfkomma= 4;                   // 4 Kommastellen fuer %k my_printf

  printf("\n\r Pseudokomma: 42.16 * 13.24 = %k", a2);

  // Ausgabe in - txtbuffer - fuer spaetere Verwendung
  sprintf(txtbuffer, "\n\n\r von sprintf: 42.16 * 13.24 = %k", a2);

  z= 74;
  printf("\n\n\r Zahlenwert %d entspricht: %ddez, 0x%x, Ascii-Zeichen '%c'",z,z,z,z);

  a2= 0x3a4bf2c4;
  printf("\n\n\r 32-Bitwert: %D dez = %X hex", a2, a2 );

  // den Text aus txtbuffer ausgeben
  putstr(txtbuffer);

  counter= 0;
  printf("\n\n\r");
  while(1)
  {
    printf("    counter= %d    \r",counter);
    counter++;
    delay(1000);
  }
}
