/* ------------------------------------------------------------------
                              i2c_scan_sw.c

     I2C-Busscanner mittels Software-I2C (Bitbanging)

     Scant den I2C Bus nach angeschlossenen Teilnehmern und gibt
     die Hexadresse des I2C-Device auf dem UART aus.

     Hinweis: das R-/W -Bit wird als Bestandteil der Adresse ange-
              sehen. Adressen werden also als 8-Bit Adresse
              verstanden). Somit ist dann ein Device 2 mal im Adress-
              raum vorhanden: Geradezahlige Adressen sind somit die
              "Leseadressen", ungeradzahlige die "Schreibadressen"


     Pinbelegung I2C
     ---------------
     siehe i2c_sw.h

     MCU: alle mit min. 4 kByte Flash Speicher


     27.01.2020 R. Seelig
   ------------------------------------------------------------------ */


#include <util/delay.h>
#include <avr/io.h>

#include "avr_gpio.h"
#include "i2c_sw.h"
#include "uart_all.h"
#include "my_printf.h"

#define  delay                _delay_ms
#define  printf               my_printf

#define cnt_speed    500

#define led_init()     PA1_output_init()
#define led_clr()      PA1_set()
#define led_set()      PA1_clr()



/* --------------------------------------------------------
   my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  uart_putchar(ch);
}


int main(void)
{
  uint8_t ch;
  uint8_t cx,anz;
  uint8_t ack;


  printfkomma= 3;

  uart_init();
  i2c_master_init();

  while(1)
  {
    printf("\n\rI2C Bus scanning\n\r--------------------------\n\n\r" \
           "Devices found at address:\n\n\r");

    anz= 0;

    for (cx= 0; cx< 254; cx += 2)
    {
      ack= i2c_start(cx);
      delay(1);
      i2c_stop();
      if (ack)
      {
        switch (cx)
        {
          case 0xC0 : printf("Adr. %xh : TEA5767 UKW-Radio\n\r", cx); break;
          case 0x20 :
          case 0x22 : printf("Adr. %xh : RDA5807 UKW-Radio\n\r", cx); break;
          case 0x40 :
          case 0x42 :
          case 0x44 :
          case 0x46 :
          case 0x48 :
          case 0x4A :
          case 0x4C :
          case 0x4E : printf("Adr. %xh : PCF8574 I/O Expander\n\r", cx); break;
          case 0x90 :
          case 0x92 :
          case 0x94 :
          case 0x96 :
          case 0x98 :
          case 0x9A :
          case 0x9C :
          case 0x9E : printf("Adr. %xh : LM75 Temp.-Sensor\n\r", cx); break;
          case 0xA0 :
          case 0xA2 :
          case 0xA4 :
          case 0xA6 :
          case 0xA8 :
          case 0xAA :
          case 0xAC :
          case 0xAE : printf("Adr. %xh : EEProm\n\r", cx); break;
          case 0x78 : printf("Adr. %xh : SSD13016 I2C-OLED Display\n\r", cx); break;
          case 0xD0 : printf("Adr. %xh : RTC - DS1307\n\r", cx); break;

          default   : printf("Adr. %xh : unknown\n\r",cx); break;
        }
      }
      delay(1);
    }
    printf("\n\n\rEnd of I2C-bus scanning... \n\n\r");
    i2c_stop();
    printf("Press any key for rescan... \n\r");
    ch= uart_getchar();
  }
}
