/* -----------------------------------------------
                    serial_demo2.c

     Demoprogramm fuer serielle Schnittstelle und
     einer "abgespeckten" printf-Version

     MCU : ATmega8, ATmega48/88/168/328p
     Takt: beliebig

     28.01.2020   R. Seelig

   ----------------------------------------------- */


#include <avr/io.h>
#include <string.h>

#include "my_printf.h"
#include "uart_all.h"

#define printf    my_printf
#define delay     _delay_ms


/* -----------------------------------------------------
                         f1_equal

     testet, ob 2 Gleitkommazahlen gleich gross sind.
   ----------------------------------------------------- */
bool fl_equal(float a, float b)
{
  const float epsilon = 1e-5;

  return abs(a - b) < epsilon;
}

/* -----------------------------------------------------
                         tiny_sqrt

     eigene, kuerzere, Implementierung einer Quadrat-
     wurzelfunktion (allerdings ungenauer als die
     Definition in math.h)
   ----------------------------------------------------- */
float tiny_sqrt(float value)
{
  int counter  = 0;
  float difval = value;
  float grenze = difval / 2;
  float quaval = difval * difval;
  uint8_t domul = 0;
  uint8_t dozmul = 0;

  if (value< 10)
  {
    value = value * 10000;
    dozmul = 1;
  }

  if ( (value< 100) && !(dozmul) )
  {
    value = value * 100;
    domul = 1;
  }

  while (!fl_equal(quaval,value))
  {
    if (quaval < value)
    {
      difval = difval + grenze;
    }
    else
    {
      difval = difval - grenze;
      grenze /= 2;
    }
    counter++;
    quaval = difval * difval;
  }
  if (domul)
  {
    difval= difval / 10;
    return difval;
  }
  if (dozmul)
  {
    difval= difval / 100;
  }
  return difval;
}


/* -----------------------------------------------------
                         tiny_pow

     eigene, kurze, Implementierung einer Exponential-
     funktion. Dafuer darf ist value^n n nur ein
     Integerwert erlaubt.
   ----------------------------------------------------- */
float tiny_pow(int n, float value)
{
  float tmp;

  tmp= value;
  for (int i= 0; i < n-1; i++)
  {
    tmp= tmp*value;
  }
  return tmp;
}

/* -----------------------------------------------------
                         tiny_sin

     eigene, kurze, Implementierung einer Sinusfunktion.

     Kuerzer, aber ungenauer als die in math.h
   ----------------------------------------------------- */
float tiny_sin(float value)
{

  float degree;
  float p3;
  float p5;
  float p7;
  float sinx;

  int   mflag= 0;

  while (value > 360.0f) value -= 360.0f;
  if (value > 180.0f)
  {
    mflag= - 1;
    value -= 180.0f;
  }

  if (value > 90.0f) value = 180.0f - value;

  degree= (value * M_PI) / 180.0f;

  p3 = tiny_pow(3, degree);
  p5  = tiny_pow(5, degree);
  p7  = tiny_pow(7, degree);

  sinx= (degree - (p3/6.0f) + (p5/120.0f) - (p7/5040.0f));

  if (mflag) sinx = sinx * (-1);
  return sinx;

}

/* -----------------------------------------------------
                         draw_sincurve

     zeichnet eine Sinuskurve aus Textzeichen
   ----------------------------------------------------- */
void draw_sincurve(void)
{
  for (int i= 0; i< 361; i += 10)
  {
    int x2;

    x2 = (tiny_sin(i) * 24) + 25;
    for (int x= 0; x < x2; x++) uart_putchar(' ');
    printf("o\n\r");
    delay(60);
  }
}

/* --------------------------------------------------------
   my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  uart_putchar(ch);
}


/* --------------------------------------------------------
                             main
   -------------------------------------------------------- */
int main(void)
{
  uint16_t counter = 0;
  uint16_t ch;
  float    wuwert;
  int      wuwerti;

  uart_init();

  printf("\n\n\rRS-232 fuer ATmegaxx8\n\n\r");
  printf("Protokoll 8N1 %d0 Bd\n\n\r", BAUDRATE / 10);
  printf("Ein einfacher 16 Bit-Integer Zaehler\n\r");
  printf("------------------------------------\n\n\r");

  printfkomma= 3;

  while(uart_ischar()) {ch= uart_getchar(); my_putchar(ch); }  // eventuell eingegangene Zeichen alle loeschen

  while(1)
  {
    wuwert= tiny_sin(counter);
    wuwerti= wuwert*1000;

    printf("  Counter: %xh sin(%d)= %k  \r", counter, counter, wuwerti);


    delay(500);
    counter++;
    counter = counter % 3600;

    if (uart_ischar())
    {
      ch= uart_getchar();
      printf("\n\n\rGedrueckte Taste war: %c\n\r", ch );
      printf("Beliebige Taste fuer Counterstart...\n\n\r");
      ch= uart_getchar();
      draw_sincurve();

    }

  }
}

