/* ------------------ t2313_nokia_tst.c ----------------------

   Testprogramm fuer ein Display eines Nokia Displays an
   einem ATtiny2313 MCU. Aufgrund sehr begrenzter Speicher-
   kapazitaet des Tiny's ist keine Grafik moeglich !!!

   MCU  : ATtiny2313
   Takt : 8 MHz

   28.01.2020 R. Seelig
   ---------------------------------------------------------- */

/*
                            ATTINY2313

                        Anschlusspins IC

                        +-------------+
          /reset - PA2  |  1       20 |  Vcc
             RxD - PD0  |  2       19 |  PB7 - UCSK - SCL - PCINT7
             TxD - PD1  |  3   A   18 |  PB6 - MISO - DO - PCINT6
           XTAL2 - PA1  |  4   T   17 |  PB5 - MOSI - DI - SDA - PCINT5
           XTAL1 - PA0  |  5   t   16 |  PB4 - OC1B - PCINT4
CKOUT - XCK - INT0 PD2  |  6   i   15 |  PB3 - OC1A - PCINT3
            INT1 - PD3  |  7   n   14 |  PB2 - OC0A - PCINT2
              T0 - PD4  |  8   y   13 |  PB1 - AIN1 - PCINT1
       OC0B - T1 - PD5  |  9       12 |  PB0 - AIN0 - PCINT0
                   GND  | 10       11 |  PD6 - ICP
                        +-------------+

Hinweise:

    Die Displays sind alle fuer 3,3V ausgelegt !!! Entweder wird
    der ATtiny mit 3,3V betrieben um die I/O Pins direkt mit dem
    Display verbinden zu koennen, oder es muss ein Level-Shifter
    zwischengeschaltet werden (zur Not koennen hier auch 5
    Spannungsteiler mit 1,8k zu 3,3k verwendet werden).

    Bei Verwendung von USI als Hardware SPI ist der sendende Ausgang
    D0. Das heisst: PB6 entspricht hier dann MOSI (im Gegensatz zur
    Verwendung von ISP bei der der Anschluss PB6 MISO ist) !!!!


    Hier zur Dokumentation die vorgenommene Verdrahtung angeben.
    Verdrahtung ist deklariert in t2313_n5110.h

    ATTiny2313             Display
    ------------------------------

       PB7 (19)   ---       CLK (5)
       PB6 (18)   ---       DIN (4)
       PB4 (16)   ---       DC  (3)
       PB3 (15)   ---       CE  (2)
       PB2 (14)   ---       RST (1)

*/

#include <util/delay.h>
#include <avr/io.h>

#include "t2313_n5110.h"
#include "my_printf.h"

#define  printf      tiny_printf
#define  delay       _delay_ms

#define speed     200


void my_putchar(char c)
{
  lcd_putchar_d(c);
}


int main(void)
{
  uint8_t i;

  delay(100);
  lcd_init();

  gotoxy(0,1);
  printf("printf-Ausgabe");
  printfkomma= 1;
  for (i= 100; i> 0; i--)
  {
    gotoxy(3,3);
    printf("i= %d  ", i);
    delay(100);
  }

  clrscr();


  gotoxy(0,1);
  prints(" LCD-5110\n\r ------------");
  prints("\n\r     MCU:\n\r  ATtiny2313");
  while(1)
  {
    for (i= 0; i<14; i++)
    {
      gotoxy(i,0); lcd_putchar_d('o');
      gotoxy(13-i,5); lcd_putchar_d('o');
      delay(speed);
      gotoxy(i,0); lcd_putchar_d(' ');
      gotoxy(13-i,5); lcd_putchar_d(' ');
    }
    for (i= 1; i< 5; i++)
    {
      gotoxy(13,i); lcd_putchar_d('o');
      gotoxy(0,5-i); lcd_putchar_d('o');
      delay(speed);
      gotoxy(13,i); lcd_putchar_d(' ');
      gotoxy(0,5-i); lcd_putchar_d(' ');

    }
  }

}
