/* -------------------------------------------------
                    tftmono_demo.c

   Testprogramm fuer monochrome, grafikfaehige
   Displays.

   Unterstuetzte Displays sind:

   - SSD1306 (I2C und SPI Interface)
   - PC8544 Displays (Nokia 5110 Display)

   MCU       :  ATmega8, ATmega48 .. ATmega328p
   Takt      :  nicht spezifiziert

   IDE       : keine (Editor / make)
   Toolchain : AVR-GCC

   07.05.19 by R. Seelig
  -------------------------------------------------- */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#include "avr_gpio.h"

#include "tftmono.h"
#include "my_printf.h"

#include "female84.h"

#define  delay                _delay_ms
#define  printf               my_printf

/* --------------------------------------------------------
   putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  lcd_putchar(ch);
}

/* ------------------------------------------------------------------------
                                    MAIN
   ------------------------------------------------------------------------ */

int main(void)
{
  uint8_t i;

  lcd_init();
  lcd_enable();
  delay(150);

  setfont(fnt8x8);


  fb_init(64, 4);
  while(1)
  {

    clrscr();
    i= 0;
    gotoxy(0,0); printf("Counter:");
    textsize= 2;
    do
    {
      gotoxy(0,2);
      if (i< 100) printf(" ");
      if (i< 10) printf(" ");
      printf("%d",i);
      delay(50);
      i++;
    } while(i);
    textsize= 0;
    clrscr();
    setfont(fnt8x8);
    gotoxy(0,0);
    printf("Mono-TFT\n\r");
    gotoxy(1,1); printf("Line2");
    gotoxy(1,2); printf("Line3");
    gotoxy(1,3); printf("Line4");
    setfont(fnt8x8);
    delay(2000);
    clrscr();

    fb_clear();
    fb_outtextxy(1,0, 0, "Hallo Welt");

    fb_outtextxy(1,8, 0,"AVR-ATmega");

    fb_outtextxy(1,16, 0, "Line3");

    fb_outtextxy(1,24, 0,"Line4");

    fb_show(0, 0);
    delay(2000);

    fb_clear();
    fillcircle(42,20, 17, 1);
    line(0,0, 83,39, 2);
    fb_show(0, 0);
    delay(2000);

    clrscr();
    #if (ssd1306 == 1)
      showimage(0, 0, &female84_image[0], 1);           // schwarzes Display, helle Schrift
    #else
      showimage(0, 0, &female84_image[0], 2);           // helles Display, schwarze Schrift
    #endif
    delay(2000);

  }
}
