/* -------------------------------------------------------------
                      timefuncs.c

    Versuche zu Zeit und Datum nachzubilden OHNE time.h zu
    verwenden (da in glibc erst ab Version 1.8 verfuegbar)

    Hardware  : alle AVR mit genuegendem Flash-Speicher
                hier benoetigt: 2924 Bytes
    IDE       : keine (Editor / make)

    Toolchain : avr-gcc

    30.06.2024   R. Seelig
   ------------------------------------------------------------- */

#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>


#include "avr_gpio.h"
#include "my_printf.h"
#include "my_time.h"
#include "uart_all.h"

#define printf    my_printf
#define delay     _delay_ms

volatile uint32_t sekcnt;                   // Sekundenzaehler

/* --------------------------------------------------------
   my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  uart_putchar(ch);
}

/* --------------------------------------------------------
                     Interruptvector Timer1

     uebernimmt die Frequenzzaehlervariable in die Variable
     die angezeigt wird und setzt die Zaehlervariable
     zurueck
   -------------------------------------------------------- */
ISR (TIMER1_COMPA_vect)
{
  sekcnt++;
}

/* --------------------------------------------------------
                           timer1_init

     initialisiert Timer1 Compare Interrupt Modus fuer
     Aufruf des Interrupts jede Sekunde

   -------------------------------------------------------- */
void timer1_init(void)
{
  /* ---------------------------------------------
     CS10 / CS11 / CS12 sind Steuerbits
     des Taktvorteilers.

     CS 12 | CS11 | CS10
     ----------------------------------------------
       0      0      0   kein Takt, Timer gestoppt
       0      0      1   clk / 1 (kein Teiler)
       0      1      0   clk / 8
       0      1      1   clk / 64
       1      0      0   clk / 256
       1      0      1   clk / 1024
     ---------------------------------------------- */
  TCCR1B = 1 << WGM12 | 1 << CS10 | 1 << CS12;       // Taktvorteiler = 1024
  OCR1A = F_CPU / 1024;
  TCNT1 = 0;

  TIMSK1 = 1 << OCIE1A;
  sei();
}



/* -------------------------------------------------------------
                                main
   ------------------------------------------------------------- */
int main(void)
{

  mydate_time        mydate;
  uint32_t           timestamp;

  volatile uint32_t  oldsek;


  uart_init();

  printf("\n\n\rmy_time fuer ATmegaxx8\n\n\r");
  printf("Protokoll 8N1 %d0 Bd\n\n\r", BAUDRATE / 10);
  printf("------------------------------------\n\n\r");

//  timestamp= 1481559030ul;      // 12.12.2016   16:10.30
//  timestamp= 1719813788ul;      // 01.07.2024   06:03.08
  timestamp= 1719862473ul;      // 01.07.2024   19:34.33

  mydate= mydate_get(timestamp);
  printf("\n\n\r %d.%d.%d | %d:%d.%d",mydate.day, mydate.month, mydate.year, mydate.hour, mydate.min, mydate.sec);

  mydate.year= 2020;
  mydate.month= 2;
  mydate.day= 29;
  mydate.hour= 20;
  mydate.min= 12;
  mydate.sec= 17;

  timestamp= mydate_getstamp(mydate);
  mydate= mydate_get(timestamp);
  printf("\n\n\r %d.%d.%d | %d:%d.%d",mydate.day, mydate.month, mydate.year, mydate.hour, mydate.min, mydate.sec);

  timestamp += sec_per_hour;
  mydate= mydate_get(timestamp);
  printf("\n\n\r %d.%d.%d | %d:%d.%d",mydate.day, mydate.month, mydate.year, mydate.hour, mydate.min, mydate.sec);

  printf("\n\n\r");

  sekcnt= 1719862473ul;      // 01.07.2024   19:34.33
  mydate= mydate_get(sekcnt);
  oldsek= sekcnt;
  timer1_init();
  while(1)
  {
    while (oldsek == sekcnt);
    oldsek= sekcnt;

    mydate= mydate_get(sekcnt);
    printf("  %d.%d.%d | %d:%d.%d      \r",mydate.day, mydate.month, mydate.year, mydate.hour, mydate.min, mydate.sec);
  }
}

