/* ----------------------------------------------------------
                      tm16xx_demo.c

   Grundsaetzliche Funktionen des TM16xx mit 4 stelliger
   7-Segmentanzeige


     MCU   : ATmegaxx8
     F_CPU :

     Pinbelegung

     ATmegaxx8         TM16xx
     ---------------------------
        PC0            CLK
        PC1            DIO

     PB0 und PB1 sind mit jeweils 2,2 kOhm Pop-Up Wider-
     staenden nach +5V zu versehen

     04.12.2024 R. Seelig
   ---------------------------------------------------------- */

#include <util/delay.h>
#include <avr/io.h>

#include "tm16xx.h"
#include "avr_gpio.h"

#define delay     _delay_ms




/* ----------------------------------------------------------
                            MAIN
   ---------------------------------------------------------- */
void main(void)
{
  int16_t i, i2;
  uint16_t cx;
  uint8_t b;

  tm16_setbright(10);
  tm16_init();

  tm16_setdez6digit(765432,1);
  delay(1500);
  tm16_clear();
  tm16_xofs= 2;
  while(1)
  {

    for (i= 10; i> -1; i--)               // Countdown-Zaehler
    {
      tm16_setdez(i,3);                   // Dezimalpunkt an
      _delay_ms(500);
      tm16_setdez(i,0);                   // Dezimalpunkt aus
      _delay_ms(500);
    }

    for (i= 0; i< 3; i++)                 // 0000 blinken lassen
    {
      tm16_clear();
      _delay_ms(500);
      tm16_setdez(0,0);
      _delay_ms(500);
    }

    for (i= 3; i> -1; i--)                // Ziffer wandern lassen
    {
      tm16_clear();
      tm16_setzif(i,0);
      _delay_ms(300);
    }

    tm16_sethex(0x1b3a);                  // Hex-Wert anzeigen
    _delay_ms(2000);

    tm16_clear();
    for (i2= 0; i2< 5; i2++)             // Segmentlauflicht, 20 Durchlaeufe
    {
      for (i= 0; i< 6; i++)
      {
        tm16_setseg(0,i);
        tm16_setseg(3,i);
        _delay_ms(100);
      }
    }

    cx= 0;
    // Tasten als Code anzeigen, Taste 3 wiederholt Demo sofort
    // ansonsten wird das Demo nach einem "Timeout" wiederholg
    do
    {
      b= tm16_readkey();
      if (b !=0) cx= 0;
      tm16_setdez(b,0);

      _delay_ms(50);
      cx++;
    } while ((b != 3) && (cx< 100));
    _delay_ms(50);
    while( tm16_readkey() == 3);

  }
}
